import folium as f

#from folium_vectortilelayer import VectorTileLayer
from folium_vectorgrid import VectorGridProtobuf

import json
from foliumEllipsis.util import get
from foliumEllipsis.util import  createStyleFunction
from foliumEllipsis.util import validUuid
from foliumEllipsis.util import validString

#python3 setup.py sdist bdist_wheel
#twine upload --repository pypi dist/*

__version__ = '0.0.5'
apiUrl = 'https://api.ellipsis-drive.com/v3'


def addEllipsisVectorLayer( pathId, layerName = 'layer', timestampId=None, style=None, zoom = None, token = None):
    pathId = validUuid('pathId', pathId, True )
    token = validString('token', token, False)

    metadata_url = apiUrl + '/path/' + pathId

    metadata = get(metadata_url, token)

    if type(timestampId) == type(None) or type(style) == type(None) or type(zoom) == type(None):
        if 'vector' != metadata['type']:
            raise ValueError('Given pathId is of type ' + metadata['type'] + ' but must be of type vector.')
        ts = [x for x in metadata['vector']['timestamps'] if x['status'] == 'active']
        if type(timestampId) == type(None):
            if len(ts) == 0:
                raise ValueError('This layer does not contain any active timestamps.')
            timestampId = ts[0]['id']
        t = [t for t in metadata['vector']['timestamps'] if t['id'] == timestampId  ][0]
        if t['status'] != 'active':
            raise ValueError('Given timestamp is not active')
        if not t['precompute']['hasVectorTiles']:
            raise ValueError('vector tiles are still being computed, please wait a few more moments.')
        if type(zoom) == type(None):
            zoom = t['precompute']['vectorTileZoom']
        if type(style) == type(None):
            style = [ s for s in metadata['vector']['styles'] if s['default']][0]['id']



    timestampId = validUuid('timestampId', timestampId, True )
    styleFromUuid = None
    try:
        validUuid('style', pathId, True )
        styleFromUuid = True
    except:
        try:
            json.dumps(style)
            styleFromUuid = False
        except:
            raise ValueError("style must be either a uuid or json object")

    url = apiUrl + '/ogc/mvt/' + pathId + '/{z}/{x}/{y}?timestampId=' + timestampId

    if type(token) !=type(None):
        url = url + '&token=' + token


    if styleFromUuid:
        style = [s for s in metadata['vector']['styles'] if s['id'] == style]
        if len(style) == 0:
            raise ValueError('style id not found')

        style = style[0]

    styleSheet = None
    if style['method'] == 'random':
        style_url = apiUrl +  "/ogc/mvt/" + pathId + "/styleSheet?timestampId=" + timestampId + "&style=" + json.dumps(style) + "&zoom=21"
        styleSheet = get(style_url, token)

    functionString = createStyleFunction(style, styleSheet)
    options = '''{
      "layers": ["layer"],
    "minZoom": 0,
    "maxZoom": 21,
    "minDetailZoom": 0,
    "maxDetailZoom": ''' + str(zoom) + ''',            
      "vectorTileLayerStyles": {
        "layer": ''' + functionString +  '''
      }
    }'''
    vc = VectorGridProtobuf(url, layerName, options)
    return vc



def addEllipsisRasterLayer( pathId, timestampId=None, style=None, zoom = None, token = None):

    pathId = validUuid('pathId', pathId, True )
    token = validString('token', token, False)


    if type(timestampId) == type(None) or type(style) == type(None) or type(zoom) == type(None):
        metadata_url = apiUrl + '/path/' + pathId
        metadata = get(metadata_url, token)
        if 'raster' != metadata['type']:
            raise ValueError('Given pathId is of type ' + metadata['type'] + ' but must be of type raster.')
        ts = [x for x in metadata['raster']['timestamps'] if x['status'] == 'active']
        if type(timestampId) == type(None):
            if len(ts) == 0:
                raise ValueError('This layer does not contain any active timestamps.')
            timestampId = ts[0]['id']
        t = [t for t in metadata['raster']['timestamps'] if t['id'] == timestampId  ][0]
        if t['status'] != 'active':
            raise ValueError('Given timestamp is not active')
        if type(zoom) == type(None):
            zoom = t['zoom']
        if type(style) == type(None):
            style = [ s for s in metadata['raster']['styles'] if s['default']][0]['id']



    timestampId = validUuid('timestampId', timestampId, True )

    try:
        validUuid('style', pathId, True )
    except:
        try:
            json.dumps(style)
        except:
            raise ValueError("style must be either a uuid or json object")

    url = apiUrl + '/path/' + pathId + '/raster/timestamp/' + timestampId + '/tile/{z}/{x}/{y}?style=' + style

    if type(token) !=type(None):
        url = url + '&token=' + token
    raster_layer = f.raster_layers.TileLayer(tiles=url, attr='ED', max_native_zoom=zoom)

    return raster_layer






import requests
from uuid import UUID
def validUuid(name, value, required):
    if not required and type(value) == type(None):
        return
    try:
        UUID(value, version=2)
    except:
        raise ValueError(name + ' must be of type string and be a uuid')
    return(value)

def validString(name, value, required):
    if not required and type(value) == type(None):
        return

    if type(value) != type('x'):
        raise ValueError(name + 'must be of type string')
    return(value)

def get(url, token):

    if type(token) != type(None):
        if not '?' in url:
            url = url + '?token=' + token
        else:
            url = url + '&token=' + token
    if type(token) == type(None):
        r = requests.get(url)
    else:
        r = requests.get(url,  headers = {"Authorization": 'Bearer ' + token})
    if r.status_code != 200:
        raise ValueError(r.text)

    r = r.json()
    return r


def createStyleFunction(style, styleSheet):
    if style['method'] == 'fromColorProperty':
        functionString = '''function(f) {

                if(f.color){

                return {
                  "fill": true,
                  "weight": ''' + str(style['parameters']['width']) + ''',
                  "fillColor": f.color,
                  "color": f.color,
                  "fillOpacity":''' + str(style['parameters']['alpha']) + ''',
                  "opacity":1
                };
                }else{
                return {
                  "fill": true,
                  "weight": ''' + str(style['parameters']['width']) + ''',
                  "fillColor": "''' + style['parameters']['defaultColor'] + '''",
                  "color": "''' + style['parameters']['defaultColor'] + '''",
                  "fillOpacity":''' + str(style['parameters']['alpha']) + ''',
                  "opacity":1
                };

                }



            }'''

    if style['method'] == 'random':
        arr = [x for x in styleSheet['layers'] if x['type'] == 'fill'][0]['paint']['fill-color']
        arr = arr[2:len(arr) - 1]
        p = style['parameters']['property']
        values = []
        colors = []
        for i in range(len(arr)):
            if i % 2 == 0:
                values = values + [arr[i]]
            else:
                colors = colors + [arr[i]]

        ifs = ['if( f.' + p + '==' + str(x) + ')' for x in values]

        styles = ['''{
              "fill": true,
              "weight": ''' + str(style['parameters']['width']) + ''',
              "fillColor": "''' + c + '''",
              "color": "''' + c + '''",
              "fillOpacity":''' + str(style['parameters']['alpha']) + ''',
              "opacity": 1
            }''' for c in colors]

        functionString = 'function(f){'
        i = 0
        for i in range(len(ifs)):
            functionString = functionString + ifs[i] + '{ return(' + styles[i] + ');}'

        functionString = functionString + '}'

    if style['method'] == 'singleColor':
        functionString = '''function(f) {
                return {
                  "fill": true,
                  "weight": ''' + str(style['parameters']['width']) + ''',
                  "fillColor": "''' + style['parameters']['color'] + '''",
                  "color": "''' + style['parameters']['color'] + '''",
                  "fillOpacity":''' + str(style['parameters']['alpha']) + ''',
                  "opacity":1
                };      
            }'''
    if style['method'] == 'rules':
        rules = style['parameters']['rules']

        for i in range(len(rules)):
            if rules[i]['operator'] == '=':
                rules[i]['operator'] = '==='

        ifs = ['if( f.' + x['property'] + x['operator'] + str(x['value']) + ')' for x in rules]

        styles = ['''{
              "fill": true,
              "weight": ''' + str(style['parameters']['width']) + ''',
              "fillColor": "''' + x['color'] + '''",
              "color": "''' + x['color'] + '''",
              "fillOpacity":''' + str(style['parameters']['alpha']) + ''',
              "opacity": 1
            }''' for x in rules]

        ifs = ifs + ['else']

        styles = styles + ['''{
              "fill": true,
              "weight": ''' + str(style['parameters']['width']) + ''',
              "fillColor": "''' + style['parameters']['defaultColor'] + '''",
              "color": "''' + style['parameters']['defaultColor'] + '''",
              "fillOpacity":''' + str(style['parameters']['alpha']) + ''',
              "opacity": 1
            }''']

        functionString = 'function(f){'
        i = 0
        for i in range(len(ifs)):
            functionString = functionString + ifs[i] + '{ return(' + styles[i] + ');}'

        functionString = functionString + '}'

    if style['method'] == 'transitionPoints':
        transitionPoints = style['parameters']['transitionPoints']
        p = style['parameters']['property']
        ifs = ['if( f.' + p + '<' + str(x['value']) + ')' for x in transitionPoints]
        ifs[len(ifs) - 1] = 'else'

        styles = ['''{
              "fill": true,
              "weight": ''' + str(style['parameters']['width']) + ''',
              "fillColor": "''' + x['color'] + '''",
              "color": "''' + x['color'] + '''",
              "fillOpacity":''' + str(style['parameters']['alpha']) + ''',
              "opacity": 1
            }''' for x in transitionPoints]

        functionString = 'function(f){'
        i = 0
        for i in range(len(ifs)):
            functionString = functionString + ifs[i] + '{ return(' + styles[i] + ');}'

        functionString = functionString + '}'

    functionString = functionString.replace('\n', '')
    functionString = functionString.replace(' ', '')

    return functionString



import setuptools
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="foliumEllipsis",
    version="0.0.5",
    author="Daniel van der Maas",
    author_email="daniel@ellipsis-drive.com",
    description="Package to add Ellipsis Drive layers to folium",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ellipsis-drive/folium",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    install_requires=[
    'folium',
    'folium-vectorgrid',
    ],
    python_requires='>=3.6',
)
